@@ -3,14 +3,16 @@ Helpers for the API
 """
 import re
 from collections import namedtuple
+from textwrap import dedent
+
+from parso.python.parser import Parser
+from parso.python import tree
+from parso import split_lines
 
 from jedi._compatibility import u
+from jedi.evaluate.syntax_tree import eval_atom
 from jedi.evaluate.helpers import evaluate_call_of_leaf
-from jedi import parser
-from jedi.parser import tree
-from jedi.parser import tokenize
 from jedi.cache import time_cache
-from jedi import common
 
 
 CompletionParts = namedtuple('CompletionParts', ['path', 'has_dot', 'name'])
@@ -52,7 +54,7 @@ class OnErrorLeaf(Exception):
 
 
 def _is_on_comment(leaf, position):
-    comment_lines = common.splitlines(leaf.prefix)
+    comment_lines = split_lines(leaf.prefix)
     difference = leaf.start_pos[0] - position[0]
     prefix_start_pos = leaf.get_start_pos_of_prefix()
     if difference == 0:
@@ -74,16 +76,14 @@ def _get_code_for_stack(code_lines, module_node, position):
             return u('')
 
         # If we're not on a comment simply get the previous leaf and proceed.
-        try:
-            leaf = leaf.get_previous_leaf()
-        except IndexError:
+        leaf = leaf.get_previous_leaf()
+        if leaf is None:
             return u('')  # At the beginning of the file.
 
     is_after_newline = leaf.type == 'newline'
     while leaf.type == 'newline':
-        try:
-            leaf = leaf.get_previous_leaf()
-        except IndexError:
+        leaf = leaf.get_previous_leaf()
+        if leaf is None:
             return u('')
 
     if leaf.type == 'error_leaf' or leaf.type == 'string':
@@ -95,11 +95,10 @@ def _get_code_for_stack(code_lines, module_node, position):
         # impossible.
         raise OnErrorLeaf(leaf)
     else:
-        if leaf == ';':
-            user_stmt = leaf.parent
-        else:
-            user_stmt = leaf.get_definition()
-        if user_stmt.parent.type == 'simple_stmt':
+        user_stmt = leaf
+        while True:
+            if user_stmt.parent.type in ('file_input', 'suite', 'simple_stmt'):
+                break
             user_stmt = user_stmt.parent
 
         if is_after_newline:
@@ -120,23 +119,26 @@ def get_stack_at_position(grammar, code_lines, module_node, pos):
         pass
 
     def tokenize_without_endmarker(code):
-        tokens = tokenize.source_tokens(code, use_exact_op_types=True)
+        # TODO This is for now not an official parso API that exists purely
+        #   for Jedi.
+        tokens = grammar._tokenize(code)
         for token_ in tokens:
             if token_.string == safeword:
                 raise EndMarkerReached()
             else:
                 yield token_
 
-    code = _get_code_for_stack(code_lines, module_node, pos)
+    # The code might be indedented, just remove it.
+    code = dedent(_get_code_for_stack(code_lines, module_node, pos))
     # We use a word to tell Jedi when we have reached the start of the
     # completion.
     # Use Z as a prefix because it's not part of a number suffix.
     safeword = 'ZZZ_USER_WANTS_TO_COMPLETE_HERE_WITH_JEDI'
     code = code + safeword
 
-    p = parser.ParserWithRecovery(grammar, code, start_parsing=False)
+    p = Parser(grammar._pgen_grammar, error_recovery=True)
     try:
-        p.parse(tokenizer=tokenize_without_endmarker(code))
+        p.parse(tokens=tokenize_without_endmarker(code))
     except EndMarkerReached:
         return Stack(p.pgen_parser.stack)
     raise SystemError("This really shouldn't happen. There's a bug in Jedi.")
@@ -153,7 +155,7 @@ class Stack(list):
                 yield node
 
 
-def get_possible_completion_types(grammar, stack):
+def get_possible_completion_types(pgen_grammar, stack):
     def add_results(label_index):
         try:
             grammar_labels.append(inversed_tokens[label_index])
@@ -161,17 +163,17 @@ def get_possible_completion_types(grammar, stack):
             try:
                 keywords.append(inversed_keywords[label_index])
             except KeyError:
-                t, v = grammar.labels[label_index]
+                t, v = pgen_grammar.labels[label_index]
                 assert t >= 256
                 # See if it's a symbol and if we're in its first set
                 inversed_keywords
-                itsdfa = grammar.dfas[t]
+                itsdfa = pgen_grammar.dfas[t]
                 itsstates, itsfirst = itsdfa
                 for first_label_index in itsfirst.keys():
                     add_results(first_label_index)
 
-    inversed_keywords = dict((v, k) for k, v in grammar.keywords.items())
-    inversed_tokens = dict((v, k) for k, v in grammar.tokens.items())
+    inversed_keywords = dict((v, k) for k, v in pgen_grammar.keywords.items())
+    inversed_tokens = dict((v, k) for k, v in pgen_grammar.tokens.items())
 
     keywords = []
     grammar_labels = []
@@ -205,7 +207,7 @@ def evaluate_goto_definition(evaluator, context, leaf):
     elif parent.type == 'trailer':
         return evaluate_call_of_leaf(context, leaf)
     elif isinstance(leaf, tree.Literal):
-        return context.evaluator.eval_atom(context, leaf)
+        return eval_atom(context, leaf)
     return []
 
 
@@ -244,6 +246,8 @@ def _get_call_signature_details_from_error_node(node, position):
             # until the parentheses is enough.
             children = node.children[index:]
             name = element.get_previous_leaf()
+            if name is None:
+                continue
             if name.type == 'name' or name.parent.type in ('trailer', 'atom'):
                 return CallSignatureDetails(
                     element,
@@ -255,9 +259,8 @@ def get_call_signature_details(module, position):
     leaf = module.get_leaf_for_position(position, include_prefixes=True)
     if leaf.start_pos >= position:
         # Whitespace / comments after the leaf count towards the previous leaf.
-        try:
-            leaf = leaf.get_previous_leaf()
-        except IndexError:
+        leaf = leaf.get_previous_leaf()
+        if leaf is None:
             return None
 
     if leaf == ')':
@@ -281,6 +284,8 @@ def get_call_signature_details(module, position):
 
         if node.type == 'trailer' and node.children[0] == '(':
             leaf = node.get_previous_leaf()
+            if leaf is None:
+                return None
             return CallSignatureDetails(
                 node.children[0], *_get_index_and_key(node.children, position))
 
@@ -299,7 +304,7 @@ def cache_call_signatures(evaluator, context, bracket_leaf, code_lines, user_pos
     whole = '\n'.join(other_lines + [before_cursor])
     before_bracket = re.match(r'.*\(', whole, re.DOTALL)
 
-    module_path = bracket_leaf.get_parent_until().path
+    module_path = context.get_root_context().py__file__()
     if module_path is None:
         yield None  # Don't cache!
     else:
